/**
* \file sms.h
* @brief Konfigurationsdatei zur Datei \ref sms.c
* \author Lars Neubert
* \date Feb 2010
*/

#ifndef SMS_H
#define SMS_H

#ifndef F_CPU
	#define F_CPU 8000000UL
#endif

#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <util/delay.h>



/**
* Adressat, an welche SMS geschickt werden soll (immer fhrende 0 mitschreiben, NICHT +49)
* <br> Dient zudem zur Anrufererkennung neben \ref SMS_TELNR_LIST_T.
*/
//#define SMS_TELNR "016094641455" //Thomas Ringel
#define SMS_TELNR "01701618306"	//Dirk Paukisch
//#define SMS_TELNR "015120608937" //Ronny Winkler

/**
* definiert die maximale Anzahl der Telefonnummern in der Telefonnummernliste 
* <br> Dies kann zur Anrufernummernerkennung genutzt werden.Dabei werden immer die ersten \ref SMS_TELNR_LIST_MAX Nummern der SIM-Karte verwendet (siehe \ref sms_init)
*/
#define SMS_TELNR_LIST_MAX 5

/**
* definiert die maximale Lnge einer Telefonnummer
*/
#define SMS_TELNR_MAX_LEN 15

/**
* definiert die maximale Lnge einer Textnachricht
*/
#define SMS_MESSAGE_MAX_LEN 160


/**
* Trennzeichen um Befehle aus empfangenen Textnachrichten zu selektieren
*/
#define SMS_DELIMITER ",;: "


/**
* @brief 
* Struktur mit Datums- und Zeitangaben einer empfangenen Textnachricht 
*/
typedef struct {
	uint8_t year;	/**< Jahresangabe YY */
	uint8_t month;	/**< Monatsangabe MM */
	uint8_t day;	/**< Tagangabe DD */
	uint8_t hour;	/**< Stundenangabe hh */
	uint8_t minute;	/**< Minutenangabe mm */
	uint8_t second;	/**< Sekundenangabe ss */
} DATE_T;


/**
* @brief
* Struktur zum speichern der decodierten Daten aus einer empfangenen Textnachricht
*/
typedef struct {
	char nachricht[SMS_MESSAGE_MAX_LEN+1];	/**< decodierte PDU Nachricht */
	char telnr[SMS_TELNR_MAX_LEN+1];			/**< Telefonnummer des Absenders */
	DATE_T date;			/**< Struktur mit Datums- und Zeitelementen (siehe \ref DATE_T) */
} SMS_DECODE_DATA_T; 

/**
* @brief
* Struktur zum Zuordnen eines empfangenen Kommandos zu einer Funktion
* (die konkrete Zuordnung erfolgt ber \ref sms_cmd_list in \ref sms.c) 
*/		
typedef struct SMS_CMD_LIST_S{
	char cmd[10];				/**< Zeichenkette fr Kommando */
	uint8_t (*fp)(uint8_t dat);	/**< Funktionspointer auf Funktion, welche Kommando \ref cmd zugeordnet werden soll */
}SMS_CMD_LIST_T;


/**
* @brief
* Struktur zum Verwalten von Telefonnummern, welche zur Anrufererkennung dienen
* (die konkrete Zuordnung erfolgt ber \ref sms_telnr_list in \ref sms.c bzw. \ref sms_init) 
*/	
typedef struct SMS_TELNR_LIST_S{
	char telnr[SMS_TELNR_MAX_LEN+1];
}SMS_TELNR_LIST_T;


/**
* anlegen der Struktur zum speichern der Telefonnummernliste zur Anrufererkennung in \ref sms_state_machine (Definition siehe \ref sms.h) <br>
* Das Auffllen des Arrays erfolgt in \ref sms_init.
*/
extern SMS_TELNR_LIST_T sms_telnr_list[SMS_TELNR_LIST_MAX];


/**
* globale Variable mit Telefonnummer des letzten Anrufers
*/
extern char sms_last_caller[SMS_TELNR_MAX_LEN+1];


/**
* anlegen der Struktur zum speichern der decodierten Daten der PDU Zeichenkette (Definition siehe \ref sms.h)
*/
extern SMS_DECODE_DATA_T sms_dec_data;


/**
* zusammengefasste Auflistung als Rckgabewert der State Machine (\ref sms_state_machine)
*/
typedef enum {
	SMS_OK,				/**< OK von Mobiltelefon empfangen */
	SMS_ERROR,			/**< ERROR von Mobiltelefon empgangen */
	SMS_MSG_DECODE,		/**< Textnachricht eingegangen, decodiert und in \ref SMS_DECODE_DATA_T Struktur gespeichert */
	SMS_INCOMING_RING,	/**< eingehender Anruf */
	SMS_CANCEL_RING,	/**< eingehender Anruf wurde abgewiesen */
	SMS_REQUEST_TELNR,	/**< Telefonnummer des eingehenden Anruft wurde abgefragt */
	SMS_RETRIEVE_TELNR,	/**< Anrufer wurde abgewiesen, Telefonnummer ermittelt und korrekt mit \ref SMS_TELNR bzw. \ref SMS_TELNR_LIST_T verglichen - ber \ref sms_last_caller kann jetzt der Anrufer identifiziert werden */
	SMS_RETRIEVE_nTELNR,/**< Anrufer wurde abgewiesen, Telefonnummer ermittelt und <b>nicht</b> korrekt mit \ref SMS_TELNR bzw. \ref SMS_TELNR_LIST_T verglichen - ber \ref sms_last_caller kann jetzt der Anrufer identifiziert werden */
	SMS_IDLE			/**< empfangene Zeichenkette wurde nicht in sms_state_machine "gefunden" (kein Eintrag) */
} SMS_MSG;




extern uint8_t str2int(char data);

extern void sms_init(void);
extern void sms_time( unsigned char *year,unsigned char *month,
			unsigned char *day,unsigned char *hour,
			unsigned char *minute,unsigned char *secound);
extern void sms_send(char* zielnr, char* nachricht);
extern void sms_display_text(char* nachricht);
extern void sms_decode(char* pdu, SMS_DECODE_DATA_T* data);
/*extern uint8_t sms_check_nr(char* telnr_in);
extern SMS_MSG sms_state_machine(void);


extern uint8_t sms_cmdset(uint8_t dat);*/


#endif
