/**
* \file uart.c
* @brief gibt Zugriff auf serielle Schnittstelle
* \author Lars Neubert
* \date Feb 2010
*/

#include "uart.h"


volatile uint8_t uart_str_complete=0;
volatile uint8_t uart_str_count=0;
volatile uint8_t uart_timeout=0;
volatile char uart_string[UART_MAXSTRLEN+1]={'\0'};


/**
* Initialisiert die UART Schnittstelle des Mikrocontrollers mit gewhlter Baudrate (Asynchrone 8N1).
*/
void uart_init(void)
{	
	if (UART_ISR!=0) { UCSRB = (1<<RXEN)|(1<<TXEN)|(1 << RXCIE); }	// UART TX, RX und ISR_Receive einschalten
	else {UCSRB = (1<<RXEN)|(1<<TXEN); }						// UART TX und RX einschalten 
	#ifdef __AVR_ATmegaM32__
    	UCSRC |= ( 1 << URSEL )|(1 << UCSZ1)|(1 << UCSZ0);							// Asynchron 8N1
	#else
		UCSRC |= (1 << UCSZ1)|(1 << UCSZ0);			// Asynchron 8N1
	#endif		
	UBRRH = (uint8_t) (UART_UBRR_BAUD>>8);
    UBRRL = (uint8_t) (UART_UBRR_BAUD);
	uart_str_complete=0;
} 

/**
* Sendet ein Zeichen
* @param c Zeichen welches gesendet werden soll.
* @param dummy wir lediglich bentigt, um die Ausgabe Standard Ein-/Ausgabe umzuleiten (kann NULL gesetzt werden)
*/
int uart_sendc(char c, FILE *stream)
//uint8_t uart_sendc(unsigned char c)
{
    if( c == '\n' )
        uart_sendc( '\r', stream );
 
    loop_until_bit_is_set( UCSRA, UDRE );
    UDR = c;
    return 0;
}

/**
* Wartet bis ein Zeichen verfgbar ist und gibt dieses zurck
* @param dummy wir lediglich bentigt, um die Ausgabe Standard Ein-/Ausgabe umzuleiten (kann NULL gesetzt werden)
* @return empfangenes Zeichen
*/
int uart_receive(FILE *stream)
{	
	uart_timeout=0;
    while ( !(UCSRA & (1<<RXC)) ) {  // warten bis Zeichen verfgbar

		#if UART_TIMEOUT_ON==1
		#if UART_TIMEOUT_ISR==0
		_delay_ms(1);
		uart_timeout++;
		#endif
		if (uart_timeout>UART_TIMEOUT_BOUND) {break;}
		#endif
	}
	if (uart_timeout<UART_TIMEOUT_BOUND){
    	return UDR;                    // Zeichen aus UDR zurueckgeben
	} else {
		return '\n';
	}
}


/**
* Liest ankommende Zeichen in buffer bis maxlen bzw. newline
* @param buffer fr Zeichenkette
* @param maxlen maximale Anzahl der empfangenen Zeichen (Lnge der Zeichenkette)
* @return Anzahl der empfangenen Zeichen (mit 'Stringende-Zeichen')
*/
uint8_t uart_gets(char* buffer, uint8_t maxlen)
{
	uint8_t next_char;
	uint8_t stringlen = 0;
 
    next_char = uart_receive(NULL);   // erstes Zeichen empfangen
 
	// solange Zeichen sammeln bis maxlen-1, newline oder carriage return 
  	while( (next_char!='\n') && (next_char!='\r')  && (stringlen<(maxlen-1)) ) {
    	*buffer++ = next_char;
    	stringlen++;
    	next_char = uart_receive(NULL);
  	}
  	*buffer = '\0';					// '\0' anhngen, Stringende
	return stringlen;
}


/**
* Sendet eine Zeichenkette
* @param s Pointer auf Zeichenkette
*/
void uart_sends(char *s)
{
    while (*s)
    {   uart_sendc(*s , NULL);
        s++;
    }
}

/**
* Lscht das Emfangsregister der seriellen Schnittstelle
*/
void uart_clear (void)
{
	unsigned char a;
	unsigned char c;
	for (c=0;c<10;c++){	
		a = UDR;
	}
};

/**
* Interruptroutine zum Empfang von Zeichen
* <br> Speichert empfangene Zeichen in globale Variable 'string' unter Nutzung von 'maxstrlen' und 'str_count' (siehe uart.h).
* <br> Ist eine komplette Zeichenkette empfangen, wird das globale Flag 'str_complete' gesetzt (=1, siehe uart.h).
* <br> Das Flag dient der Hauptanwendung zur Signalisierung, dass die Zeichenkette verarbeitet werden kann.
* <br> Um neue Zeichenketten zu empfangen, muss das Flag wieder rckgesetzt werden (=0).
*/


#if (UART_ISR==1)
ISR(USART_RXC_vect)
{
    unsigned char buffer;

    // Daten aus dem Puffer lesen
    buffer = UDR;
	if ( uart_str_complete==0 ){	// wenn uart_string gerade in Verwendung, neues Zeichen verwerfen
		// Daten werden erst in string geschrieben, wenn nicht String-Ende/max Zeichenlnge erreicht ist/string gerade verarbeitet wird
	  	if (buffer!='\n' && buffer!='\r' && uart_str_count<UART_MAXSTRLEN-1){ //&& uart_str_complete!=0) {
			uart_string[uart_str_count]=buffer;
			uart_str_count++;
		} else {
			uart_string[uart_str_count]='\0';
			uart_str_count=0;
			uart_str_complete=1;
		}
	}
}
#endif
